/* -*- mode: c++; tab-width: 4; c-basic-offset: 4 -*- */
/*
 * Copyright (C) 2005 Atmark <atmarkat _AT_ msn _DOT_ com>
 *                    AGAWA Koji <i _AT_ atty _DOT_ jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <qapplication.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qmessagebox.h>
#include <qlayout.h>

#include <QTreeWidget>
#include <QTreeWidgetItem>
#include <QTreeWidgetItemIterator>

#include "hotkeyconfig.h"
#include "configuration.h"
#include "keynames.h"
#include "action.h"
#include "identifykeydialog.h"

//! ホットキー一覧のために属性を追加した QTreeWidgetItem です。
class HotKeyListItem : public QTreeWidgetItem
{
public:
	HotKeyListItem(int context, int key, const Action& action, QTreeWidget *parent)
		: QTreeWidgetItem(parent),
		  context_(context),
		  key_(key),
		  action_(action)
	{
		static const char *contextNames[] = {
			QT_TR_NOOP("Global"),
			QT_TR_NOOP("Playlist"),
			QT_TR_NOOP("Video"),
			QT_TR_NOOP("Audio / PlayInfo")
		};
		setText(0, KeyNames::getName(key));
		setText(1, HotKeyConfig::tr(contextNames[context]));
		setText(2, action.translatedName());
	}

	int context() const { return context_; }
	int key() const { return key_; }
	const Action& action() const { return action_; }

private:
	int context_;
	int key_;
	const Action& action_;
};

HotKeyConfig::HotKeyConfig(Configuration& config, QWidget *parent)
	: ConfigWidget(config, parent),
	  allActions_(gActionManager.allActions()),
	  currentContext_(Configuration::HOTKEY_CONTEXT_GLOBAL),
	  lastKey_(-1)
{
}

void HotKeyConfig::setStatesByConfiguration(const Configuration& config)
{
	ConfigWidget::setStatesByConfiguration(config);

	readFromConfiguration(config);
}

void HotKeyConfig::readFromConfiguration(const Configuration& config)
{
	list_->clear();

	for (int i = 0; i < Configuration::HOTKEY_CONTEXT_HOTAREAS; ++i) {
		for (Configuration::HotKeyMap::ConstIterator it = config.hotKeyMap[i].begin();
			 it != config.hotKeyMap[i].end();
			 ++it)
		{
			const Action& action(gActionManager.actionByAccelID(it.value()));
			new HotKeyListItem(i, it.key(), action, list_);
		}
	}

	list_->resizeColumnToContents(0);
	list_->resizeColumnToContents(1);
	list_->resizeColumnToContents(2);
}

#include <qtabbar.h>

void HotKeyConfig::createWidgets()
{
	ConfigWidget::createWidgets();

	QVBoxLayout *mainLayout = new QVBoxLayout(this);

	actionCombo_ = new QComboBox(this);

	for (int i = 0; i < allActions_.count(); ++i)
		actionCombo_->addItem(allActions_.at(i)->translatedName());

	mainLayout->addWidget(actionCombo_);

	QHBoxLayout *buttonLayout = new QHBoxLayout(this);

	QComboBox *contextList = new QComboBox(this);
	contextList->addItems(QStringList()
		<< tr("Global")
		<< tr("Playlist")
		<< tr("Video")
		<< tr("Audio / PlayInfo")
	);
	connect(contextList, SIGNAL(activated(int)), this, SLOT(setContext(int)));
	keyButton_ = new QPushButton(tr("Scan key..."), this);
	connect(keyButton_, SIGNAL(clicked()), this, SLOT(identifyKey()));
	QPushButton *addButton = new QPushButton(tr("Add"), this);
	connect(addButton, SIGNAL(clicked()), this, SLOT(addHotKey()));
	QPushButton *removeButton = new QPushButton(tr("Remove"), this);
	connect(removeButton, SIGNAL(clicked()), this, SLOT(removeHotKey()));
	buttonLayout->addWidget(contextList);
	buttonLayout->addWidget(keyButton_);
	buttonLayout->addWidget(addButton);
	buttonLayout->addWidget(removeButton);
	mainLayout->addLayout(buttonLayout);

	// List
	list_ = new QTreeWidget(this);
	list_->setAllColumnsShowFocus(true);
	list_->setColumnCount(3);
	list_->setHeaderLabels(QStringList()
		<< tr("Key")
		<< tr("Context")
		<< tr("Action")
	);
	mainLayout->addWidget(list_);

	QPushButton *revertToDefaultHotkeysButton = new QPushButton(tr("Revert to default hotkeys"), this);
	connect(revertToDefaultHotkeysButton, SIGNAL(clicked()), this, SLOT(revertToDefaultHotkeys()));
	mainLayout->addWidget(revertToDefaultHotkeysButton);
}

void HotKeyConfig::identifyKey()
{
	IdentifyKeyDialog dialog(this);
	if (dialog.exec() == QDialog::Accepted) {
		int key = dialog.key();
		if (key > 0) {
			keyButton_->setText(QString(tr("Key:")) + KeyNames::getName(key));
			lastKey_ = key;
		}
	}
}

//! 新しいホットキーを追加します。
void HotKeyConfig::addHotKey()
{
	const Action& action(*(allActions_.at(actionCombo_->currentIndex())));

	if (lastKey_ < 0)
		return;

	Configuration::HotKeyMap::Iterator it(config().hotKeyMap[currentContext_].find(lastKey_));
	if (it != config().hotKeyMap[currentContext_].end()) {
		int ret =
			QMessageBox::warning(this, tr("Replace"),
								 tr("Do you want to replace the existing hotkey?"),
								 QMessageBox::Yes, QMessageBox::Cancel);
		if (ret == 1)
			return;

		config().hotKeyMap[currentContext_].erase(it);

		for (QTreeWidgetItemIterator it(list_); *it; ++it) {
			HotKeyListItem *item = static_cast<HotKeyListItem *>(*it);
			if (currentContext_ == item->context() &&
				lastKey_ == item->key()) {
				delete item;
			}
		}
	}

	config().hotKeyMap[currentContext_].insert(lastKey_, action.accelID());

	new HotKeyListItem(currentContext_, lastKey_, action, list_);
}

//! 選択されたホットキーを削除します。
void HotKeyConfig::removeHotKey()
{
	HotKeyListItem *item = static_cast<HotKeyListItem *>(list_->currentItem());

	if (item) {
		if (config().hotKeyMap[item->context()].contains(item->key())) {
			config().hotKeyMap[item->context()].remove(item->key());
			delete item;
		}
	}
}

void HotKeyConfig::revertToDefaultHotkeys()
{
	int ret;
	ret = QMessageBox::warning(this, tr("Confirmation"),
							   tr("Revert to default hotkeys?"),
							   tr("Revert"), tr("Cancel"), QString::null,
							   1, 1);
	if (ret == 0)
	{
		Configuration workingConfiguration_ = Configuration();

		for (int i = Configuration::HOTKEY_CONTEXT_GLOBAL; i < Configuration::HOTKEY_CONTEXT_HOTAREAS; ++i)
			config().hotKeyMap[i] = workingConfiguration_.hotKeyMap[i];

		readFromConfiguration(workingConfiguration_);
	}
}
